#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;

// referencing this breaks the possibility to connect join and split ?!
//using VVVV.Nodes.Generic;


using SlimDX;
using SlimDX.Direct3D11;

using FeralTic.DX11;
using FeralTic.DX11.Resources;


#endregion usings

namespace VVVV.DX11.Nodes
{
	public class MyContainer
	{
		public RGBAColor color;
		public DX11Resource<DX11Texture2D> texture;
		
		public MyContainer(RGBAColor col, DX11Resource<DX11Texture2D> tex)
		{
			this.color = col;
			this.texture = tex;
		}
	}
	
	
	#region PluginInfo
	[PluginInfo(Name = "MyContainerJoin", Category = "MyContainer", Help = "Basic template with one value in/out", Tags = "c#")]
	#endregion PluginInfo
	public class ContainerJoin : IPluginEvaluate
	{
		#region fields & pins
		[Input("Color")]
		public ISpread<RGBAColor> FColor;

		[Input("Texture")]
		public ISpread<DX11Resource<DX11Texture2D>> FTexture;

		[Output("Output")]
		public ISpread<MyContainer> FOutput;

		[Import()]
		public ILogger FLogger;
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			FOutput.SliceCount = SpreadMax;

			for (int i = 0; i < SpreadMax; i++)
				FOutput[i] = new MyContainer(FColor[i], FTexture[i]);

			//FLogger.Log(LogType.Debug, "hi tty!");
		}
	}
	
	#region PluginInfo
	[PluginInfo(Name = "MyContainerSplit", Category = "MyContainer", Help = "Basic template with one value in/out", Tags = "c#")]
	#endregion PluginInfo
	public class ContainerSplit : IPluginEvaluate
	{
		[Input("Input")]
		public ISpread<MyContainer> FInput;

		#region fields & pins
		[Output("Color")]
		public ISpread<RGBAColor> FColor;

		[Output("Texture")]
		public ISpread<DX11Resource<DX11Texture2D>> FTexture;

		
		[Import()]
		public ILogger FLogger;
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			FColor.SliceCount = SpreadMax;
			FTexture.SliceCount = SpreadMax;

			
			for (int i = 0; i < SpreadMax; i++)
			{
				FColor[i] = FInput[i].color;
				FTexture[i] = FInput[i].texture;
			}
			//FLogger.Log(LogType.Debug, "hi tty!");
		}
	}
	
	
	/*
	[PluginInfo(Name = "Cons",
                Category = "MyContainer",
                Help = "Concatenates all input spreads to one output spread.",
                Tags = "generic, spreadop"
                )]
    public class MyContainerConsNode : DX11ResourceConsNode<MyContainer> 
	{

    }
	*/

}
